"""Built-in plugins for prefact.

This directory contains the official plugins that ship with prefact.
"""

__plugin_api_version__ = "1.0.0"
__plugin_name__ = "builtin"
__version__ = "0.1.10"
__description__ = "Built-in prefact plugins"
__author__ = "Pprefact Team"

# Rules will be registered when the plugin is loaded
rules = []
